'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _ = require('lodash');
var React = require('react');
var PropTypes = require('./prop-types');

var options = void 0;

var wrapPropTypes = function wrapPropTypes() {
  // Adds a .type key which allows the type to be derived during the
  // evaluation process. This is necessary for complex types which
  // return the result of a generator function, leaving no way to
  // determine which type instantiated it.

  var original = _.cloneDeep(PropTypes);

  _.each(PropTypes, function (v, k) {
    if (v.isRequired !== undefined) {
      // Simple type. Just extend the object
      _.defaultsDeep(PropTypes[k], { type: k, isRequired: { type: k } });
    } else {
      // Complex type. Must extend the creator's return value
      PropTypes[k] = function (arg) {
        return _.defaultsDeep(original[k](arg), {
          type: k, arg: arg,
          isRequired: { type: k, arg: arg }
        });
      };
    }
  });
};

wrapPropTypes();

var GENERATORS = {
  // Simple types
  array: function array(propName) {
    return [propName];
  },
  bool: function bool() {
    return true;
  },
  func: function func() {
    return function () {};
  },
  number: function number() {
    return 1;
  },
  object: function object(propName) {
    return _defineProperty({}, propName, propName);
  },
  string: function string(propName) {
    return propName;
  },
  any: function any(propName) {
    return propName;
  },
  element: function element(propName) {
    return React.createElement('div', propName);
  },
  node: function node(propName) {
    return propName;
  },

  // Complex types
  arrayOf: function arrayOf(propName, type) {
    return [generateOneProp(type, propName, false)];
  },
  instanceOf: function instanceOf(propName, klass) {
    return new klass();
  },
  objectOf: function objectOf(propName, type) {
    return { key: generateOneProp(type, propName, false) };
  },
  oneOf: function oneOf(propName, values) {
    return _.first(values);
  },
  oneOfType: function oneOfType(propName, types) {
    return forceGenerateOneProp(_.first(types), propName);
  },
  shape: function shape(propName, _shape) {
    return generateProps(_shape, options);
  }
};

var shouldGenerate = function shouldGenerate(propType) {
  return (
    // Generate required props, and this is the required version
    options.required && !propType.isRequired ||
    // Generate optional props, and this is the optional version
    options.optional && !!propType.isRequired
  );
};

var generateOneProp = function generateOneProp(propType, propName) {
  var wrapInArray = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

  var generate = options.generators[propType.type].bind(undefined, propName);
  var arg = propType.arg;
  if (generate) {
    if (shouldGenerate(propType)) {
      if (wrapInArray) {
        return [propName, generate(arg)];
      } else {
        return generate(arg);
      }
    }
  }
};

var forceGenerateOneProp = function forceGenerateOneProp(propType, propName) {
  var generate = GENERATORS[propType.type].bind(undefined, propName);
  var arg = propType.arg;
  if (generate) {
    return generate(arg);
  }
};

var generateProps = function generateProps(arg, opts) {
  options = _.defaults({}, opts, { required: true, optional: false });
  if (opts && opts.generators) {
    options.generators = _.defaults({}, opts.generators, GENERATORS);
  } else {
    options.generators = GENERATORS;
  }

  var propTypes = void 0;

  if (!arg) {
    throw new TypeError('generateProps expected a propType object or a React Component');
  } else if (_.isPlainObject(arg.propTypes)) {
    propTypes = arg.propTypes;
  } else if (_.isPlainObject(arg)) {
    propTypes = arg;
  } else {
    throw new TypeError('generateProps expected a propType object or a React Component');
  }

  return _(propTypes).map(generateOneProp).compact().fromPairs().value();
};

module.exports = generateProps;